@@ -2,26 +2,20 @@
 multipatt <- function (x, cluster, func = "IndVal.g", duleg = FALSE, restcomb=NULL, nperm = 999,                                                                  
   torus = FALSE, grid.size, print.perm = FALSE)                                                                                              
 {
-	
-### Internal functions
-# Matrix of cluster membership
-vector.to.partition <- function(v, clnames) {
-    m <- t(sapply(v,function(x) as.numeric(x==clnames)))
-    dimnames(m) = list(1:length(v),clnames)
-    return(m)                                                                                                                              
-}                                                                                                                                          
+	                                                                                                                              
 
 # Matrix of possible cluster combinations
-cl.comb <- function(k) {
+cl.comb <- function(clnames) {
+	  k <- length(clnames)
     ep <- diag(1,k,k)
-    names.ep <- 1:k
+    names.ep <- clnames
     for(j in 2:k) {
       nco <- choose(k,j)
       co <- combn(k,j)
       epn <- matrix(0,ncol=nco,nrow=k)
       for(i in 1:ncol(co)) {
 	  epn[co[,i],i] <- 1
-	  names.ep <- c(names.ep, paste(co[,i], collapse = "+"))
+	  names.ep <- c(names.ep, paste(clnames[co[,i]], collapse = "+"))
       }
       ep <- cbind(ep,epn)
     }
@@ -30,7 +24,7 @@ cl.comb <- function(k) {
 }
 
 # Correlation measures for combinations
-rcomb <- function(x, cluster, comb,  clnames, k, mode="group", duleg=FALSE, restcomb=NULL) {
+rcomb <- function(x, cluster, comb, k, mode="group", duleg=FALSE, restcomb=NULL) {
       nsps = ncol(x)
       N = length(cluster)	
       ni = diag(t(comb) %*% comb)[1:k]
@@ -80,14 +74,14 @@ rcomb <- function(x, cluster, comb,  clnames, k, mode="group", duleg=FALSE, rest
   num = N*aspC - aspK%o%nC
   den = sqrt(((N*lspK)-aspK^2)%o%(N*nC-(nC^2)))
   str=num/den
-  if(duleg & !is.null(restcomb)) str <- str[,restcomb]
   if(!duleg) str <- str[,-ncol(str)] # remove all sites as combination for correlation indices
+  if(!duleg && !is.null(restcomb)) str <- str[,restcomb]
 #  colnames(str) <- colnames(comb)[1:ncol(str)]
   return(str)
 }
 
 # IndVal for combinations
-indvalcomb <- function(x, cluster, comb, clnames, k, mode = "group", duleg = FALSE) {
+indvalcomb <- function(x, cluster, comb, k, mode = "group", duleg = FALSE, restcomb=NULL) {
   tx <- t(x)
   aisp = tx %*% comb
   dx <- dim(tx)
@@ -95,21 +89,24 @@ indvalcomb <- function(x, cluster, comb, clnames, k, mode = "group", duleg = FAL
 #    nisp = t(ifelse(x > 0, 1, 0)) %*% comb
   ni = diag(t(comb) %*% comb)
   nispni = sweep(nisp, 2, ni, "/")   
-  if (mode == "site") A = sweep(aisp, 1, colSums(x), "/")  else {
+  if (mode == "site") A = sweep(aisp, 1, colSums(x), "/")  
+  else {
     aispni = sweep(aisp[, 1:k], 2, ni[1:k], "/")
     asp = rowSums(aispni[, 1:k])
-    if(duleg) A = sweep(aispni, 1, asp, "/") else {
+    if(duleg) A = sweep(aispni, 1, asp, "/") 
+    else {
     s = aispni #matrix(0, nrow = nrow(aispni), ncol = ncol(comb))
     for(j in 2:(k)) {
       co <- combn(k,j)
       sn <- apply(co, 2, function(x) rowSums(aispni[,x]))
       s <- cbind(s, sn)
     }
-  } 
-  A = sweep(s, 1, asp, "/")
+	  A = sweep(s, 1, asp, "/")
+  	} 
   }
   iv = sqrt(A * nispni)
   colnames(iv) <- colnames(comb)
+  if(!duleg && !is.null(restcomb)) iv = iv[,restcomb]
   return(iv)
 }
 
@@ -142,28 +139,29 @@ sampletorus<-function(grid.size) {
 	if(k<2) stop("At least two clusters are required.")
   if(sum(is.na(cluster))>0) stop("Cannot deal with NA values. Remove and run again.")
   if(sum(is.na(x))>0) stop("Cannot deal with NA values. Remove and run again.")
-  if(!is.null(restcomb) && !is.integer(restcomb)) stop("'restcomb' must be a vector of integer values.") 
+  if(!is.null(restcomb)) restcomb = as.integer(restcomb)
 
   # creates combinations from clusters
-  if (duleg) comb <- vector.to.partition(cluster, clnames) else {
-      combin <- cl.comb(k)	# possible combinations (can also be used for permutations)
-      #Builds the plot membership matrix corresponding to combinations
-      comb <- combin[cluster,]
-      }
+  combin <- cl.comb(clnames)	# possible combinations (can also be used for permutations)
+  if (duleg) combin <- combin[,1:k]
+  
+  #Builds the plot membership matrix corresponding to combinations
+  comb <- combin[cluster,]
 
   # Computes association strength for each group
   str <- switch(func,
-    r =		rcomb(x, cluster, comb, clnames, k, mode = "site", duleg = duleg, restcomb = restcomb),
-    r.g = 	rcomb(x, cluster, comb, clnames, k, mode = "group", duleg = duleg, restcomb = restcomb),
-    IndVal = 	indvalcomb(x, cluster, comb, clnames, k, mode = "site", duleg),
-    IndVal.g = 	indvalcomb(x, cluster, comb, clnames, k, mode = "group", duleg),
+    r =		rcomb(x, cluster, comb, k, mode = "site", duleg = duleg, restcomb = restcomb),
+    r.g = 	rcomb(x, cluster, comb, k, mode = "group", duleg = duleg, restcomb = restcomb),
+    IndVal = 	indvalcomb(x, cluster, comb, k, mode = "site", duleg, restcomb = restcomb),
+    IndVal.g = 	indvalcomb(x, cluster, comb, k, mode = "group", duleg, restcomb = restcomb),
     )
 
   # Maximum association strength
   maxstr = apply(str,1,max) 
   wmax <- max.col(str)
   #prepares matrix of results
-  m <- as.data.frame(t(combin[,wmax]))
+  if(!duleg && !is.null(restcomb))  m <- as.data.frame(t(combin[,restcomb][,wmax]))
+  else  m <- as.data.frame(t(combin[,wmax]))
   dimnames(m) <- list(vegnames, sapply(clnames, function(x) paste("s", x, sep='.')))
   m$index <- wmax
   m$stat <- apply(str,1,max)
@@ -173,13 +171,12 @@ sampletorus<-function(grid.size) {
   for (p in 1:nperm) {
       if (!torus) pInd <- sample(1:length(cluster)) else pInd <- sampletorus(grid.size)
       tmpcls = cluster[pInd]
-    if (duleg)  combp = vector.to.partition(tmpcls, clnames) else
-		combp = combin[tmpcls,]
+		 combp = combin[tmpcls,]
       tmpstr <- switch(func,
-	r   = rcomb(x, cluster = tmpcls, combp, clnames, k, mode = "site", duleg = duleg, restcomb = restcomb),
-	r.g = rcomb(x, cluster = tmpcls, combp, clnames, k, mode = "group", duleg = duleg, restcomb = restcomb),
-	IndVal = indvalcomb(x, cluster = tmpcls, combp, clnames, k, mode = "site", duleg),
-	IndVal.g= indvalcomb(x, cluster = tmpcls, combp, clnames, k, mode = "group", duleg)
+	r   = rcomb(x, cluster = tmpcls, combp, k, mode = "site", duleg = duleg, restcomb = restcomb),
+	r.g = rcomb(x, cluster = tmpcls, combp, k, mode = "group", duleg = duleg, restcomb = restcomb),
+	IndVal = indvalcomb(x, cluster = tmpcls, combp, k, mode = "site", duleg, restcomb = restcomb),
+	IndVal.g= indvalcomb(x, cluster = tmpcls, combp, k, mode = "group", duleg, restcomb = restcomb)
       )
       tmpmaxstr <- vector(length=nrow(tmpstr))
       for(i in 1:nrow(tmpstr)) tmpmaxstr[i] <- max(tmpstr[i,])	# apply is more slowly in this case
@@ -190,6 +187,8 @@ sampletorus<-function(grid.size) {
   #Put NA for the p-value of species whose maximum associated combination is the set of all combinations
   m$p.value[m$index == (2^k-1)] <- NA
 
+  if(!duleg && !is.null(restcomb))  comb<-comb[,restcomb]
+  
   a = list(call=match.call(), func = func, cluster = cluster, comb = comb, str = str, sign = m)
   class(a) = "multipatt"
   return(a)
